# this forge file is split out from root.py so that projects built with forge
# can use this configuration information
import os
import sys

forge = sys.modules["forge"]

forge.cprint(forge.YELLOW,0,'Free Electron Common forge.py support')

# remain here for reference until win32 is tested
def deprecated_setup_boost():
	forge.boostlibs = ['filesystem', 'regex', 'thread']
	forge.boostlib = {}
	forge.boost_version = os.getenv('BOOST_VERSION')
	if not forge.boost_version:
		forge.boost_version = '1_30'
	if forge.boost_version >= '1_30':
		if forge.api == "x86_win32":
			forge.cprint(forge.RED,1,'not really tested yet')
		for l in forge.boostlibs:
			forge.boostlib[l] = 'boost_' + l
			if forge.api == "x86_win32" and forge.codegen == 'debug':
				forge.boostlib[l] += 'd'
	else:
		if forge.api == "x86_win32":
			forge.boostlibs = ['thread'] # others are auto-linked
			boost_prefix = 'boost_'
			boost_compiler = '-vc90'
			boost_threading = '-mt'
			boost_debug = ''
			if forge.codegen == 'debug':
				boost_debug = '-gd'
		else:
			boost_prefix = 'boost_'
			boost_compiler = '-gcc'
			boost_threading = '-mt'
			boost_debug = ''
			if forge.codegen == 'debug':
				boost_debug = '-d'
		for l in forge.boostlibs:
			forge.boostlib[l] = boost_prefix + l + boost_compiler + boost_threading + boost_debug + '-' + forge.boost_version

	boost_include = os.getenv('BOOST_INCLUDE')
	forge.linkmap['boost'] = ''
	if boost_include:
		boost_lib = os.getenv('BOOST_LIB')
		if not boost_lib:
			forge.cprint(forge.RED,1,'BOOST_LIB required with BOOST_INCLUDE')
			sys.exit(1)
		boost_structure = os.getenv('BOOST_STRUCTURE')
		if not boost_structure:
			forge.cprint(forge.RED,1,'BOOST_STRUCTURE required with BOOST_INCLUDE')
			forge.cprint(forge.RED,0,'  bin-stage - for local boost build')
			forge.cprint(forge.RED,0,'  install   - libraries are just flat in BOOST_LIB')
			sys.exit(1)
		if boost_structure == 'bin-stage':
			forge.linkmap['boost'] = forge.libincl + os.path.join( boost_lib, 'python','build','bin-stage')
			forge.linkmap['boost'] += ' ' + forge.libincl + os.path.join( boost_lib, 'thread','build','bin-stage')
			forge.linkmap['boost'] += ' ' + forge.libincl + os.path.join( boost_lib, 'filesystem','build','bin-stage')
		elif boost_structure == 'install':
			forge.linkmap['boost'] = forge.libincl + boost_lib
		any_header_path = os.path.join(boost_include,'boost','any.hpp')
		if os.path.exists(any_header_path):
			forge.includemap['boost'] = boost_include
			forge.includeScanSuppress.append('boost')
			forge.cprint(forge.CYAN,0,'BOOST_INCLUDE: ' + boost_include)
		else:
			forge.cprint(forge.RED,1,'BOOST_INCLUDE not set properly.')
			forge.cprint(forge.RED,0,'"%s" not found.' % (any_header_path))

	cooked_boost_libs = []
	for l in forge.boostlib.keys():
		cooked_boost_libs += [ forge.boostlib[l] ]
	add_libs('boost', cooked_boost_libs)

def setup_boost():
	forge.boostlibs = ['filesystem', 'thread']
	if forge.regex:
		forge.boostlibs += ['regex']
	forge.boostlib = {}
	forge.boost_version = os.getenv('BOOST_VERSION')
	if not forge.boost_version:
		forge.boost_version = '1_34'
	if forge.api == "x86_win32":
		forge.cprint(forge.RED,1,'**********************************************************')
		forge.cprint(forge.RED,1,'new setup_boost -- untested on win32 -- see common.py')
		forge.cprint(forge.RED,1,'**********************************************************')
#		forge.boostlibs = ['thread']	# others are auto-linked
		forge.boostlibs = []			# auto-linked
		boost_prefix = 'boost_'
		boost_compiler = '-vc90'
		boost_threading = '-mt'
		boost_debug = ''
		if forge.codegen == 'debug':
			boost_debug = '-gd'
		lib_suffix = ".lib"
	elif forge.api == "ppc_osx" or forge.api == "x86_osx":
		forge.cprint(forge.RED,1,'**********************************************************')
		forge.cprint(forge.RED,1,'new setup_boost -- still testing on osx -- see common.py')
		forge.cprint(forge.RED,1,'**********************************************************')
		boost_prefix = 'boost_'
		boost_compiler = '-gcc'
		boost_threading = '-mt'
		boost_debug = ''
		if forge.codegen == 'debug':
			boost_debug = '-d'
		lib_suffix = ".dylib"
	else:
		boost_prefix = 'boost_'
		boost_compiler = '-gcc'
		boost_threading = '-mt'
		boost_debug = ''
		if forge.codegen == 'debug':
			boost_debug = '-d'
		lib_suffix = ".so"
	for l in forge.boostlibs:
		lib_prefix = os.getenv('BOOST_LIB')
		if not lib_prefix:
			if forge.api == "x86_win32":
				lib_prefix = "C:/Program Files/boost/boost_" + forge.boost_version + "/lib/"
			else:
				lib_prefix = "/usr/lib/"
		else:
			if forge.api == "x86_win32":
				lib_prefix += "/"
		if not forge.api == "x86_win32":
			lib_prefix = os.path.join(lib_prefix, "lib")

		attempts = []

		filename = ""
		if forge.codegen == 'debug':
			filename = boost_prefix + l + boost_compiler + boost_threading + boost_debug + '-' + forge.boost_version
			attempts.append(filename)
			if not os.path.exists(lib_prefix + filename + lib_suffix):
				filename = boost_prefix + l + boost_compiler + boost_threading + boost_debug
				attempts.append(filename)

		if not os.path.exists(lib_prefix + filename + lib_suffix):
			filename = boost_prefix + l + boost_compiler + boost_threading + '-' + forge.boost_version
			attempts.append(filename)
		if not os.path.exists(lib_prefix + filename + lib_suffix):
			filename = boost_prefix + l + boost_compiler + boost_threading
			attempts.append(filename)
		if not os.path.exists(lib_prefix + filename + lib_suffix):
			filename = boost_prefix + l + boost_threading + '-' + forge.boost_version
			attempts.append(filename)
		if not os.path.exists(lib_prefix + filename + lib_suffix):
			filename = boost_prefix + l + boost_threading
			attempts.append(filename)
		if not os.path.exists(lib_prefix + filename + lib_suffix):
			filename = boost_prefix + l + '-' + forge.boost_version
			attempts.append(filename)
		if not os.path.exists(lib_prefix + filename + lib_suffix):
			filename = boost_prefix + l
			attempts.append(filename)
		if not os.path.exists(lib_prefix + filename + lib_suffix):
			forge.cprint(forge.RED,1,"failed to assemble a lib name for " + l)
			for a in attempts:
				forge.cprint(forge.GREEN,1,"  tried "+lib_prefix + a + lib_suffix)

		if os.path.exists(lib_prefix + filename + lib_suffix):
			forge.boostlib[l] = filename
			forge.cprint(forge.CYAN,0,"BOOST: using " +
					lib_prefix + filename + lib_suffix)
		else:
			forge.failed = 1

	boost_include = os.getenv('BOOST_INCLUDE')
	forge.linkmap['boost'] = ''
	if boost_include:
		boost_lib = os.getenv('BOOST_LIB')
		if not boost_lib:
			forge.cprint(forge.RED,1,'BOOST_LIB required with BOOST_INCLUDE')
			sys.exit(1)
		boost_structure = os.getenv('BOOST_STRUCTURE')
		if not boost_structure:
			forge.cprint(forge.RED,1,'BOOST_STRUCTURE required with BOOST_INCLUDE')
			forge.cprint(forge.RED,0,'  bin-stage - for local boost build')
			forge.cprint(forge.RED,0,'  install   - libraries are just flat in BOOST_LIB')
			sys.exit(1)
		if boost_structure == 'bin-stage':
			forge.linkmap['boost'] = forge.libincl + os.path.join( boost_lib, 'python','build','bin-stage')
			forge.linkmap['boost'] += ' ' + forge.libincl + os.path.join( boost_lib, 'thread','build','bin-stage')
			forge.linkmap['boost'] += ' ' + forge.libincl + os.path.join( boost_lib, 'filesystem','build','bin-stage')
		elif boost_structure == 'install':
			forge.linkmap['boost'] = forge.libincl + boost_lib
		any_header_path = os.path.join(boost_include,'boost','any.hpp')
		if os.path.exists(any_header_path):
			forge.includemap['boost'] = boost_include
			forge.includeScanSuppress.append('boost')
			forge.cprint(forge.CYAN,0,'BOOST_INCLUDE: ' + boost_include)
		else:
			forge.cprint(forge.RED,1,'BOOST_INCLUDE not set properly.')
			forge.cprint(forge.RED,0,'"%s" not found.' % (any_header_path))

	cooked_boost_libs = []
	for l in forge.boostlib.keys():
		cooked_boost_libs += [ forge.boostlib[l] ]
	add_libs('boost', cooked_boost_libs)

def platform_setup():
	if forge.api == "x86_linux" or forge.api == "x86_64_linux":
		forge.fe_os = 'FE_LINUX'
		forge.fe_osver = '2'
		forge.fe_compiler = 'FE_GNU'
		forge.fe_gl2d = 'FE_2D_X_GFX'
		forge.fe_gl3d = 'FE_3D_OPENGL'
		forge.fe_hw = 'FE_X86'
		forge.includemap['include'] = '/usr/X11R6/include'
		forge.linkmap['linux'] = '-lm -lc -lcrypt -ldl'
		forge.winlibs = '-L/usr/X11/lib -L/usr/X11R6/lib -lX11 -lXext'
		if forge.api == "x86_64_linux":
			forge.winlibs = '-L/usr/X11R6/lib64 ' + forge.winlibs
		forge.gllibs = '-lGL'
		forge.gllibs += ' -lGLU'
		forge.cppmap['checks'] = '-ansi'
		forge.linkmap['checks'] = '-ansi'
		forge.cppmap['efficiency'] = '-pipe'
		forge.linkmap['efficiency'] = '-pipe'
		forge.cppmap['multithread'] = '-D_REENTRANT -D_POSIX_REENTRANT_FUNCTIONS -DZT_POSIX'
		forge.linkmap['multithread'] = '-lpthread -lrt'
		forge.cppmap['template-depth'] = '-ftemplate-depth-60'
	elif forge.api == "ppc_osx" or forge.api == "x86_osx":
		forge.fe_os = 'FE_OSX'
		forge.fe_osver = '10'
		forge.fe_compiler = 'FE_GNU'
#		forge.fe_gl2d = 'FE_2D_OSX'
		forge.fe_gl2d = 'FE_2D_X_GFX'
		forge.fe_gl3d = 'FE_3D_OPENGL'
		forge.fe_hw = 'FE_PPC'
		forge.includemap['include'] = '/usr/X11R6/include'
		forge.linkmap['osx'] = '-lm -lc -ldl'
		forge.winlibs = '-L/usr/X11R6/lib -lX11 -lXext'
#		forge.gllibs = '-framework OpenGL -framework GLUT'
		forge.gllibs += ' -lGL -lGLU'
		forge.cppmap['checks'] = '-ansi'
		forge.linkmap['checks'] = '-ansi'
		forge.cppmap['efficiency'] = '-pipe'
		forge.linkmap['efficiency'] = '-pipe'
		forge.cppmap['multithread'] = '-D_REENTRANT -D_POSIX_REENTRANT_FUNCTIONS -DZT_POSIX'
		forge.linkmap['multithread'] = '-lpthread'
		forge.cppmap['template-depth'] = '-ftemplate-depth-60'

	elif forge.api == "x86_cygwin":
		forge.fe_os = 'FE_CYGWIN'
		forge.fe_osver = '2'
		forge.fe_compiler = 'FE_GNU'
		forge.fe_gl2d = 'FE_2D_X_GFX'
		forge.fe_gl3d = 'FE_3D_OPENGL'
		forge.fe_hw = 'FE_X86'
		forge.includemap['include'] = '/usr/X11R6/include'
		forge.linkmap['linux'] = '-lm -lc -lcrypt -ldl'
		forge.winlibs = '-L/usr/X11/lib -L/usr/X11R6/lib -lX11 -lXext'
		forge.gllibs = '-lGL -lGLU'
		forge.cppmap['multithread'] = ' -DREENTRANT -DZT_POSIX'
		forge.linkmap['multithread'] += '-lpthread -lrt'

	elif forge.api == "x86_win32":
		forge.fe_os = 'FE_WIN32'
		forge.fe_osver = '4'
		forge.fe_compiler = 'FE_MICROSOFT'
		forge.fe_gl2d = 'FE_2D_GDI'
		forge.fe_gl3d = 'FE_3D_OPENGL'
		forge.fe_hw = 'FE_X86'
		forge.linkmap['win32'] = 'kernel32.lib user32.lib gdi32.lib winspool.lib comdlg32.lib advapi32.lib shell32.lib ole32.lib oleaut32.lib uuid.lib winmm.lib ws2_32.lib'
		forge.winlibs = ''
		forge.gllibs = 'opengl32.lib'
		forge.gllibs += ' glu32.lib'
		forge.cprint(forge.RED,1,'downgrading warning level from /W3 to /W1 due to boost slop')
		forge.cppmap['win32_warning'] = '/W1'

	forge.apiCodegen = forge.api + "_" + forge.codegen

def compiler_setup():
	compiler = os.getenv('FE_CC')
	if compiler:
		compiler_base = os.path.basename(compiler)
#		forge.cxx = compiler		overwrite setting from forge.py
		if compiler_base == 'icc':
			forge.fe_compiler = 'FE_INTEL'
		if compiler_base == 'dmc':
			forge.cxx = 'C:\\dm\\bin\\sc.exe'
			forge.fe_compiler = 'FE_DMC'

	if forge.fe_compiler == 'FE_INTEL':
		forge.cppmap['codegen'] = ''
		if forge.codegen == 'debug':
			forge.cppmap['intel'] = '-g'
		forge.linkmap['intel_cc'] = '-lcxa -lcprts -lunwind'

	if forge.fe_compiler == 'FE_MICROSOFT':
		if forge.codegen == 'debug':
			forge.cppmap['fe_codegen'] = '-DFE_CONSOLE_LOG'
#			forge.linkmap['win32_debug'] = '/NODEFAULTLIB:libcmt.lib libcmtd.lib'

	if forge.fe_compiler == 'FE_DMC':
		forge.cprint(forge.RED,1,'Digital Mars build incomplete. Awaiting full boost support')
		sys.exit(1)
		forge.objout = '-o'
		forge.exeout = '-o'
		forge.dlflags = ' -WD'
		if forge.codegen == 'debug':
			forge.cppmap['codegen'] = '-D_DEBUG -g'
			forge.dlflags += ' /DEBUG'
		elif forge.codegen == 'optimize':
			forge.cppmap['codegen'] = '-DNDEBUG'
		else:
			forge.cppmap['codegen'] = ''
		#forge.cppmap['codegen'] += ' -Aa -Ab -Ae -Ar -Aw -C -EC -Jb -Pz -3 -mn'
		forge.cppmap['codegen'] += ' -A- -Aa -Ae -Ar'
		forge.libincl = '-L'
		forge.nolink = '-c'
		forge.linker = 'c:\\dm\\bin\dmc.exe'
		forge.linkerflags = ''
		forge.ar = 'c:\\dm\\bin\\lib.exe'
		forge.arflags = '-p32'
		forge.aroutprefix = '-c '
		forge.objsuf = '.obj'
		forge.exesuf = '.exe'
		forge.libflag = ''
		forge.libpre = ''
		forge.libsuf = '.lib'
		forge.libdl = '.dll'
		forge.pymodsuf = '.pyd'
		forge.ld1 = ''
		forge.ld2 = ''
		forge.forcecxx = '-cpp'
		forge.fat1 = ''
		forge.fat2 = ''

		forge.winlibs = ''
		forge.gllibs = ''

		forge.cppmap['win32'] =''
		forge.cppmap['win32_warning'] = ''

		forge.includemap['stl'] = 'C:\dm\stlport\stlport'
		forge.linkmap['win32'] = 'kernel32.lib user32.lib gdi32.lib comdlg32.lib advapi32.lib ole32.lib oleaut32.lib uuid.lib'
		#forge.includemap['dmc'] = 'C:\dm\include'

	forge.gfxlibs = forge.winlibs + ' ' + forge.gllibs

def add_libs(name, libs):
	if not forge.linkmap.has_key(name):
		forge.linkmap[name] = ''
	for l in libs:
		if forge.api == "x86_linux" or forge.api == "x86_64_linux" or forge.api == "ppc_osx" or forge.api == "x86_osx":
			forge.linkmap[name] += ' ' + forge.libflag + l
		else:
			forge.linkmap[name] += ' ' + forge.libpre + l + forge.libsuf

