/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "datatool/datatool.h"

namespace fe
{

Config::Config(void)
{
}

Config::~Config(void)
{
}

void Config::configSet(const String &a_key, Instance &a_instance)
{
	m_catalog[a_key] = a_instance;
}

bool Config::configLookup(const String &a_key, Instance &a_instance)
{
	bool rv = false;
	// lookup local
	std::map<String, Instance>::iterator i_v = m_catalog.find(a_key);
	if(i_v == m_catalog.end())
	{
		// recursive parent check
		if(m_parent.isValid())
		{
			rv = m_parent->configLookup(a_key, a_instance);
		}
		else
		{
			rv = false;
		}
	}
	else
	{
		a_instance = i_v->second;
		rv = true;
	}

	// lookup in the registry catalog
	if(!rv)
	{
		rv = registry()->master()->catalogLookup(a_key, a_instance);
	}

	return rv;
}

#if 0
void Config::cfgSet(const String &a_key, const String &a_value)
{
	m_dictionary[a_key] = a_value;
//maybe a callback type of thing here to enable cacheing
}

const String Config::cfg(const String &a_key)
{
	String value;
	if(!cfgLookup(a_key, value))
	{
		feX("Config::cfg", "\'%s\' not configured for \'%s\'", a_key.c_str(),
			name().c_str());
	}
	return value;
}

bool Config::cfgLookup(const String &a_key, String &a_value)
{
	std::map<String, String>::iterator i_value =
		m_dictionary.find(a_key);
	if(i_value == m_dictionary.end())
	{
		if(m_parent.isValid())
		{
			return m_parent->cfgLookup(a_key, a_value);
		}
		return false;
	}
	a_value = i_value->second;
	return true;
}

const String Config::cfg(const String &a_key, const String &a_default)
{
	String value;
	if(!cfgLookup(a_key, value))
	{
		return a_default;
	}
	return value;
}

void Config::cfg(std::vector< std::pair<String,String> > &a_entries)
{
	for(std::map<String, String>::iterator i_scp = m_dictionary.begin();
		i_scp != m_dictionary.end(); i_scp++)
	{
		a_entries.push_back(
			std::pair<String,String>(i_scp->first,i_scp->second));
	}
}
#endif

#if 1
void Config::configParent(sp<ConfigI> a_parent)
{
	m_parent = a_parent;
}
#endif

} /* namespace */

