/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __draw_ViewOpenGL_h__
#define __draw_ViewOpenGL_h__

namespace fe
{

/**************************************************************************//**
	@brief OpenGL-specific implementations for ViewI

	@ingroup draw
*//***************************************************************************/
class FE_DL_EXPORT ViewOpenGL: virtual public ViewI
{
	public:
						ViewOpenGL();

						//* as ViewI
virtual	void			setWindow(sp<WindowI> spWindowI)
						{	m_spWindowI=spWindowI; }
virtual	sp<WindowI>		window(void) const				{ return m_spWindowI; }

virtual	void			setViewMode(sp<ViewMode> spViewMode)
						{	m_spViewMode=spViewMode; }
virtual	sp<ViewMode>	viewMode(void) const			{ return m_spViewMode; }

virtual	void					use(ViewMode::Projection a_projection);
virtual	ViewMode::Projection	projection(void) const;

virtual	Real			pixelSize(const SpatialVector& center,
								Real radius) const;
virtual	Real			worldSize(const SpatialVector& center,
								Real pixels) const;

virtual	SpatialVector	unproject(Real winx,Real winy,Real winz,
								ViewMode::Projection a_projection) const;
virtual	SpatialVector	project(const SpatialVector& point,
								ViewMode::Projection a_projection) const;

virtual	void			screenAlignment(SpatialTransform& a_rotation);

virtual	void			screenInfo(Real &a_screenScale,
							SpatialVector &a_screenLocation,
							const SpatialTransform& a_rotation,
							const SpatialVector &a_location);

	private:
		bool			currentContextIsSet(void);

		void			useOrtho(void);
		void			usePerspective(void);

		Matrix<4,4,Real>	callOrtho(Real a_zoom,
								const Vector<2,Real> &a_center,
								const Vector<2,Real> &a_size) const;
		void			prepare(ViewMode::Projection a_projection,
								GLdouble* model16,
								GLdouble* proj16,GLint* view) const;

		sp<WindowI>				m_spWindowI;
		sp<ViewMode>			m_spViewMode;
		ViewMode::Projection	m_projection;
};

} // namespace

#endif /* __draw_ViewOpenGL_h__ */
