/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __math_DAGNode_h__
#define __math_DAGNode_h__

namespace fe
{

#if FE_CODEGEN<=FE_DEBUG
#define FE_DAG_VALIDATE()	validate()
#else
#define FE_DAG_VALIDATE()
#endif

/**************************************************************************//**
	@brief Node in a Directed Acyclic Graph

	@ingroup math
*//***************************************************************************/
class FE_DL_EXPORT DAGNode
{
	public:
						DAGNode(void)										{}
virtual					~DAGNode(void)										{}

		void			attachTo(DAGNode& rNode);
		void			detachFrom(DAGNode& rNode);
		void			detach(void);

		void			validate(void);

		List<DAGNode*>::Iterator	parent(void){ return m_parents; }
		List<DAGNode*>::Iterator	child(void)	{ return m_children; }

		U32				parentCount(void)		{ return m_parents.size(); }
		U32				childCount(void)		{ return m_children.size(); }

	private:
		List<DAGNode*>	m_parents;
		List<DAGNode*>	m_children;
};

inline void DAGNode::validate(void)
{
	//* verify that each parent references this node exactly once
	List<DAGNode*>::Iterator iterator(m_parents);
	DAGNode* pParent;
	while((pParent= *iterator++)!=NULL)
	{
		List<DAGNode*>::Iterator iterator2(pParent->m_children);
		DAGNode* pChild;
		U32 count=0;
		while((pChild= *iterator2++)!=NULL)
			if(pChild==this)
				count++;
		if(count!=1)
			feX("DAGNode::validate","found %d parent references",count);
	}
}

inline void DAGNode::attachTo(DAGNode& rNode)
{
	FE_DAG_VALIDATE();
	ListCore::Context context;
	if(rNode.m_children.searchForElement(context,&rNode))
	{
		feLog("DAGNode::attachTo node already attached\n");
		return;
	}

	rNode.m_children.append(this);
	m_parents.append(&rNode);
	FE_DAG_VALIDATE();
}

inline void DAGNode::detachFrom(DAGNode& rNode)
{
	FE_DAG_VALIDATE();
	rNode.m_children.remove(this);
	m_parents.remove(&rNode);
	FE_DAG_VALIDATE();
}

inline void DAGNode::detach(void)
{
	FE_DAG_VALIDATE();
	List<DAGNode*>::Iterator iterator(m_parents);
	DAGNode* pParent;
	while((pParent= *iterator++)!=NULL)
		detachFrom(*pParent);
	m_parents.removeAll();
	FE_DAG_VALIDATE();
}

} /* namespace */

#endif /* __math_DAGNode_h__ */
