/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "window/window.h"
#include "draw/draw.h"

using namespace fe;


int main(int argc,char** argv)
{
	UNIT_START();

	BWORD complete=FALSE;
	U32 limit=(argc>1)? atoi(argv[1])+1: 0;

	try
	{
		sp<Master> spMaster(new Master);
		sp<Registry> spRegistry=spMaster->registry();

		Result result=spRegistry->manage("fexNativeWindowDL");
		UNIT_TEST(successful(result));

		result=spRegistry->manage("fexDrawOpenGLDL");
		UNIT_TEST(successful(result));

		{
			sp<WindowI> spWindowI(spRegistry->create("WindowI"));
			sp<WindowI> spWindowI2(spRegistry->create("WindowI"));
			sp<HandlerI> spHandler1(spRegistry->create("*.LogWindowHandler"));
			sp<HandlerI> spHandler2(spRegistry->create("*.LogWindowHandler"));
			sp<SignalerI> spSignalerI(spRegistry->create("SignalerI"));
			sp<DrawI> spDrawI(spRegistry->create("DrawI"));
			sp<Scope> spScope(spRegistry->create("Scope"));

			if(!spWindowI.isValid() || !spWindowI2.isValid() ||
					!spScope.isValid() || !spSignalerI.isValid())
				feX(argv[0], "can't continue");

			UNIT_TEST(spHandler1.isValid() && spHandler2.isValid());

			sp<Layout> spBeatLayout=spScope->declare("BEAT");
			spSignalerI->insert(spWindowI->getEventContextI(),spBeatLayout);
			Record beat=spScope->createRecord(spBeatLayout);

			sp<Layout> spEventLayout=
					spScope->lookupLayout(FE_EVENT_LAYOUT);
			UNIT_TEST(spEventLayout.isValid());

			if(spHandler1.isValid())
			{
				spSignalerI->insert(spHandler1,spEventLayout);
			}
			if(spHandler2.isValid())
			{
				sp<SignalerI> spWindowSignaler=spWindowI;
				UNIT_TEST(spWindowSignaler.isValid());
				if(spWindowSignaler.isValid())
					spWindowSignaler->insert(spHandler2,spEventLayout);
			}

			sp<EventContextI> spEventContextI=spWindowI->getEventContextI();
			if(spEventContextI.isValid() && spDrawI.isValid())
			{
				spDrawI->setFontBase(spEventContextI->fontBase());
			}

			spWindowI->open("NativeWindow");
			spWindowI2->open("NativeWindow 2");

			U32 count=0;
			while(++count!=limit)
			{
				if(limit)
				{
					feLog("Frame %d\n",count);
				}
				spSignalerI->signal(beat);

				for(U32 pass=0;pass<2;pass++)
				{
					sp<WindowI> spThisWindowI= pass? spWindowI2: spWindowI;

					spThisWindowI->makeCurrent();
					spThisWindowI->clear();

					if(spDrawI.isValid())
					{
						const Color pink(1.0f,0.5f,0.5f);
						const Vector3f corner(0.1f,0.1f,0.0f);

						String text;
						text.sPrintf("window %d %p",pass,spThisWindowI.raw());

						spDrawI->drawAlignedText(corner,text.c_str(),pink);
						spDrawI->drawAxes(0.2f);
					}

					spThisWindowI->swapBuffers();
				}

				milliSleep(1);
			}
		}

		complete=TRUE;
	}
	catch(Exception &e)
	{
		e.log();
	}
	catch(...)
	{
		feLog("uncaught exception\n");
	}

	UNIT_TEST(complete);
	UNIT_TEST(!Counted::trackerCount());
	UNIT_TRACK(6);
	UNIT_RETURN();
}
