/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __solve_BlockPCG_h__
#define __solve_BlockPCG_h__

namespace fe
{

class Preconditioner
{
	public:
virtual			~Preconditioner(void){}
virtual	void	apply(		std::vector<SpatialVector> &a_r,
							const std::vector<SpatialVector> &a_x) const	= 0;
};

class BlockDiagonalPreconditioner : public Preconditioner
{
	public:
				BlockDiagonalPreconditioner(void);
virtual			~BlockDiagonalPreconditioner(void);

virtual	void	apply(		std::vector<SpatialVector> &a_r,
							const std::vector<SpatialVector> &a_x) const;

		std::vector<SpatialMatrix> &diagonal(void);

	private:
		std::vector<SpatialMatrix>	m_diagonal;
};

class FilterConstraint
{
	public:
						FilterConstraint(void);
virtual					~FilterConstraint(void);

		const SpatialMatrix	&filter(void) const;
		void				setIndex(unsigned long a_index);
		unsigned long		index(void) const;

		void			makePointConstraint(void);
		void			makeLineConstraint(const SpatialVector &a_direction);
		void			makePlaneConstraint(const SpatialVector &a_normal);

	private:
		SpatialMatrix	m_filter;
		unsigned long	m_index;


};

class BlockPCG
{
	public:
				BlockPCG(void);
virtual			~BlockPCG(void);

		bool	solve(		std::vector<SpatialVector> &a_x,
							const sp<SparseMatrix3x3> a_A,
							const std::vector<SpatialVector> &a_b,
							const Preconditioner &a_P,
							const std::vector<FilterConstraint> &a_filters);

		const std::vector<SpatialVector> &residual(void) const;

	private:
		Real	inner(		const std::vector<SpatialVector> &a_left,
							const std::vector<SpatialVector> &a_right);

		void	residual(	std::vector<SpatialVector> &a_r,
							const sp<SparseMatrix3x3> a_A,
							const std::vector<SpatialVector> &a_x,
							const std::vector<SpatialVector> &a_b);

		void	setSize(	unsigned long a_size);

		void	applyFilters(std::vector<SpatialVector> &a_r,
							const std::vector<FilterConstraint> &a_filters);

		Real						m_tol;
		unsigned int				m_maxIterations;
		std::vector<SpatialVector>	m_r;
		std::vector<SpatialVector>	m_q;
		std::vector<SpatialVector>	m_d;
		std::vector<SpatialVector>	m_s;
		std::vector<SpatialVector>	m_tmp0;
		std::vector<SpatialVector>	m_tmp1;
		unsigned int				m_n;

};

} /* namespace */

#endif /* __solve_BlockPCG_h__ */

