/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "solve/solve.h"

using namespace fe;

int main(void)
{
	UNIT_START();
	BWORD completed=FALSE;

	try
	{
		SpatialTransform A;
		SpatialTransform B;

		setIdentity(A);
		rotate(A,30.0f*degToRad,e_xAxis);
		translate(A,SpatialVector(100.0f,200.0f,300.0f));
		rotate(A,60.0f*degToRad,e_yAxis);

		feLog("matrix B = sqrt A\n");

		MatrixSqrt<SpatialTransform> matrixSqrt;
		matrixSqrt.solve(B,A);

		SpatialTransform C=A-B*B;

		F32 sum=0.0f;
		for(U32 m=0;m<width(C);m++)
		{
			for(U32 n=0;n<height(C);n++)
			{
				sum+=fabs(C(m,n));
			}
		}

		feLog("\nA\n%s\n",print(A).c_str());
		feLog("\nB\n%s\n",print(B).c_str());
		feLog("\nB*B\n%s\n",print(B*B).c_str());
		feLog("\nC=A-B*B\n%s\n",print(C).c_str());

		feLog("\nC component sum = %.6G\n",sum);
		UNIT_TEST(sum<2e-3);

		completed=TRUE;
	}
	catch(Exception &e)	{ e.log(); }
	catch(...)			{ feLog("uncaught exception\n"); }

	UNIT_TEST(completed);
	UNIT_TRACK(2);
	UNIT_RETURN();
}
