/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "solve/solve.h"

#define ELEMENTS	7

using namespace fe;

int main(void)
{
	UNIT_START();
	BWORD completed=FALSE;

	try
	{
		{
			SparseMatrix<F32>	A(ELEMENTS,ELEMENTS);
			DenseVector<F32>	x(ELEMENTS);

			setIdentity(A);
			setAll(x,1.0f);

			A(1,2)=2.0f;
			x[4]=4.0f;

			DenseVector<F32> b=A*x;

			feLog("A=\n%s\n",print(A).c_str());
			feLog("x=%s\n",print(x).c_str());
			feLog("b=%s\n",print(b).c_str());

			// 1 3 1 1 4 1 1
			DenseVector<F32>	expected(ELEMENTS);

			setAll(expected,1.0f);
			expected[1]=3.0f;
			expected[4]=4.0f;

			feLog("expected=%s\n",print(expected).c_str());
			UNIT_TEST(equivalent(b,expected,1e-3f));
		}

		{
			UpperTriangularBCRS A;
			//FullBCRS A;
			A.startrow();

			SpatialMatrix block;
			setAll(block, 0.0);

			block(0,0) = 1.0;
			block(1,1) = 2.0;
			block(2,2) = 3.0;

			A.insert(0,block);

			block(0,1) = 2.3;
			block(0,2) = 2.4;
			block(1,0) = -3.4;
			block(1,2) = 3.4;
			block(2,0) = -2.4;
			block(2,1) = -5.4;

			A.insert(1,block);
			A.insert(2,block);

			A.startrow();

			A.insert(1,block);
			A.insert(2,block);

			A.startrow();

			block(2,0) = -20.4;
			A.insert(2,block);

			A.done();

			std::vector<SpatialVector> x,b;

			x.push_back(SpatialVector(2.0,3.0,4.0));
			x.push_back(SpatialVector(2.0,3.0,4.0));
			x.push_back(SpatialVector(2.0,3.0,4.0));

			b.resize(3);

			A.multiply(b,x);

			Real sum = 0;
			for(unsigned int i=0; i < b.size(); i++)
			{
				feLog("%f %f %f\n", b[i][0], b[i][1], b[i][2]);
				sum += b[i][0];
				sum += b[i][1];
				sum += b[i][2];
			}
			feLog("sum=%f\n", sum);
			UNIT_TEST(isZero(sum - (Real)90.1));
		}

		completed=TRUE;
	}
	catch(Exception &e)	{ e.log(); }
	catch(...)			{ feLog("uncaught exception\n"); }

	UNIT_TEST(completed);
	UNIT_TRACK(3);
	UNIT_RETURN();
}
