/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "window.pmh"

namespace fe
{

WindowEvent::Mask WindowEvent::stringToMask(const String &a_string)
{
	WindowEvent::Mask mask(
		WindowEvent::e_sourceNull,
		WindowEvent::e_itemNull,
		WindowEvent::e_stateNull);

	boost::char_separator<char> cat_sep(" ,");
	boost::char_separator<char> item_sep(" |");

	std::string source_str;
	std::string item_str;
	std::string state_str;
	std::vector< std::string > fields;

	std::string std_str = a_string.c_str();
	typedef boost::tokenizer<boost::char_separator<char> > t_tokenizer;
	t_tokenizer tokens(std_str, cat_sep);
	for(t_tokenizer::iterator i_t = tokens.begin();
					i_t != tokens.end(); ++i_t)
	{
		fields.push_back((*i_t));
	}

	if(fields.size() == 1)
	{
		source_str = "keyboard";
		item_str = fields[0];
		state_str = "press";
	}
	else if(fields.size() == 3)
	{
		source_str = fields[0];
		item_str = fields[1];
		state_str = fields[2];
	}
	else
	{
		feX("WindowEvent::stringToMask",
			"invalid mask: wrong field count: [%s]", a_string.c_str());
	}


	t_tokenizer source_tokens(source_str, item_sep);
	for(t_tokenizer::iterator i_t = source_tokens.begin();
					i_t != source_tokens.end(); ++i_t)
	{
		if((*i_t) == "mousePosition")
		{
			mask.orSource(WindowEvent::e_sourceMousePosition);
		}
		if((*i_t) == "mouseButton")
		{
			mask.orSource(WindowEvent::e_sourceMouseButton);
		}
		if((*i_t) == "keyboard")
		{
			mask.orSource(WindowEvent::e_sourceKeyboard);
		}
		if((*i_t) == "system")
		{
			mask.orSource(WindowEvent::e_sourceSystem);
		}
		if((*i_t) == "user")
		{
			mask.orSource(WindowEvent::e_sourceUser);
		}
		if((*i_t) == "joy0")
		{
			mask.orSource(WindowEvent::e_sourceJoy0);
		}
		if((*i_t) == "any")
		{
			mask.orSource(WindowEvent::e_sourceAny);
		}
	}

	t_tokenizer item_tokens(item_str, item_sep);
	for(t_tokenizer::iterator i_t = item_tokens.begin();
					i_t != item_tokens.end(); ++i_t)
	{
		if((*i_t) == "any")
		{
			mask.orItem(WindowEvent::e_itemAny);
		}
		if((*i_t) == "move")
		{
			mask.orItem(WindowEvent::e_itemMove);
		}
		if((*i_t) == "drag")
		{
			mask.orItem(WindowEvent::e_itemDrag);
		}
		if((*i_t) == "left")
		{
			mask.orItem(WindowEvent::e_itemLeft);
		}
		if((*i_t) == "middle")
		{
			mask.orItem(WindowEvent::e_itemMiddle);
		}
		if((*i_t) == "right")
		{
			mask.orItem(WindowEvent::e_itemRight);
		}
		if((*i_t) == "wheel")
		{
			mask.orItem(WindowEvent::e_itemWheel);
		}
		if((*i_t) == "backspace")
		{
			mask.orItem(WindowEvent::e_keyBackspace);
		}
		if((*i_t) == "tab")
		{
			mask.orItem(WindowEvent::e_keyTab);
		}
		if((*i_t) == "lineFeed")
		{
			mask.orItem(WindowEvent::e_keyLineFeed);
		}
		if((*i_t) == "carriageReturn")
		{
			mask.orItem(WindowEvent::e_keyCarriageReturn);
		}
		if((*i_t) == "space")
		{
			mask.orItem(WindowEvent::e_keySpace);
		}
		if((*i_t) == "escape")
		{
			mask.orItem(WindowEvent::e_keyEscape);
		}
		if((*i_t) == "delete")
		{
			mask.orItem(WindowEvent::e_keyDelete);
		}
		if((*i_t) == "asciiMask")
		{
			mask.orItem(WindowEvent::e_keyAsciiMask);
		}
		if((*i_t) == "cursor")
		{
			mask.orItem(WindowEvent::e_keyCursor);
		}
		if((*i_t) == "cursorUp")
		{
			mask.orItem(WindowEvent::e_keyCursorUp);
		}
		if((*i_t) == "cursorDown")
		{
			mask.orItem(WindowEvent::e_keyCursorDown);
		}
		if((*i_t) == "cursorLeft")
		{
			mask.orItem(WindowEvent::e_keyCursorLeft);
		}
		if((*i_t) == "cursorRight")
		{
			mask.orItem(WindowEvent::e_keyCursorRight);
		}
		if((*i_t) == "cursorMask")
		{
			mask.orItem(WindowEvent::e_keyCursorMask);
		}
		if((*i_t) == "shift")
		{
			mask.orItem(WindowEvent::e_keyShift);
		}
		if((*i_t) == "capLock")
		{
			mask.orItem(WindowEvent::e_keyCapLock);
		}
		if((*i_t) == "control")
		{
			mask.orItem(WindowEvent::e_keyControl);
		}
		if((*i_t) == "modifiers")
		{
			mask.orItem(WindowEvent::e_keyModifiers);
		}
		if((*i_t) == "used")
		{
			mask.orItem(WindowEvent::e_keyUsed);
		}
		if((*i_t) == "idle")
		{
			mask.orItem(WindowEvent::e_itemIdle);
		}
		if((*i_t) == "resize")
		{
			mask.orItem(WindowEvent::e_itemResize);
		}
		if((*i_t) == "expose")
		{
			mask.orItem(WindowEvent::e_itemExpose);
		}
		if((*i_t) == "closeWindow")
		{
			mask.orItem(WindowEvent::e_itemCloseWindow);
		}
		if((*i_t) == "quit")
		{
			mask.orItem(WindowEvent::e_itemQuit);
		}
		if((*i_t) == "poll")
		{
			mask.orItem(WindowEvent::e_itemPoll);
		}
		if((*i_t) == "clear")
		{
			mask.orItem(WindowEvent::e_windowClear);
		}
		if((*i_t) == "swapBuffers")
		{
			mask.orItem(WindowEvent::e_windowSwapBuffers);
		}
		if((*i_t) == "leftStickX")
		{
			mask.orItem(WindowEvent::e_joyLeftStickX);
		}
		if((*i_t) == "leftStickY")
		{
			mask.orItem(WindowEvent::e_joyLeftStickY);
		}
		if((*i_t) == "leftPadX")
		{
			mask.orItem(WindowEvent::e_joyLeftPadX);
		}
		if((*i_t) == "leftPadY")
		{
			mask.orItem(WindowEvent::e_joyLeftPadY);
		}
		if((*i_t) == "rightStickX")
		{
			mask.orItem(WindowEvent::e_joyRightStickX);
		}
		if((*i_t) == "rightStickY")
		{
			mask.orItem(WindowEvent::e_joyRightStickY);
		}
		if((*i_t) == "start")
		{
			mask.orItem(WindowEvent::e_joyStart);
		}
		if((*i_t) == "select")
		{
			mask.orItem(WindowEvent::e_joySelect);
		}
		if((*i_t) == "leftHigh")
		{
			mask.orItem(WindowEvent::e_joyLeftHigh);
		}
		if((*i_t) == "leftLow")
		{
			mask.orItem(WindowEvent::e_joyLeftLow);
		}
		if((*i_t) == "leftDepress")
		{
			mask.orItem(WindowEvent::e_joyLeftDepress);
		}
		if((*i_t) == "rightUp")
		{
			mask.orItem(WindowEvent::e_joyRightUp);
		}
		if((*i_t) == "rightDown")
		{
			mask.orItem(WindowEvent::e_joyRightDown);
		}
		if((*i_t) == "rightLeft")
		{
			mask.orItem(WindowEvent::e_joyRightLeft);
		}
		if((*i_t) == "rightRight")
		{
			mask.orItem(WindowEvent::e_joyRightRight);
		}
		if((*i_t) == "rightHigh")
		{
			mask.orItem(WindowEvent::e_joyRightHigh);
		}
		if((*i_t) == "rightLow")
		{
			mask.orItem(WindowEvent::e_joyRightLow);
		}
		if((*i_t) == "rightDepress")
		{
			mask.orItem(WindowEvent::e_joyRightDepress);
		}
		if((*i_t) == "axes")
		{
			mask.orItem(WindowEvent::e_joyAxes);
		}
		if((*i_t) == "buttons")
		{
			mask.orItem(WindowEvent::e_joyButtons);
		}
		if((*i_t) == "joyAny")
		{
			mask.orItem(WindowEvent::e_joyAny);
		}
		if((*i_t).size() == 1)
		{
			mask.orItem((WindowEvent::Item)(*i_t).c_str()[0]);
		}
	}

	t_tokenizer state_tokens(state_str, item_sep);
	for(t_tokenizer::iterator i_t = state_tokens.begin();
					i_t != state_tokens.end(); ++i_t)
	{
		if((*i_t) == "press")
		{
			mask.orState(WindowEvent::e_statePress);
		}
		if((*i_t) == "release")
		{
			mask.orState(WindowEvent::e_stateRelease);
		}
		if((*i_t) == "repeat")
		{
			mask.orState(WindowEvent::e_stateRepeat);
		}
		if((*i_t) == "any")
		{
			mask.orState(WindowEvent::e_stateAny);
		}
		if((*i_t) == "left")
		{
			mask.orState((State)WindowEvent::e_mbLeft);
		}
		if((*i_t) == "middle")
		{
			mask.orState((State)WindowEvent::e_mbMiddle);
		}
		if((*i_t) == "right")
		{
			mask.orState((State)WindowEvent::e_mbRight);
		}
		if((*i_t) == "all")
		{
			mask.orState((State)WindowEvent::e_mbAll);
		}
		if((*i_t) == "mbAny")
		{
			mask.orState((State)WindowEvent::e_mbAny);
		}
	}

	return mask;
}

String print(const WindowEvent& event)
{
	String string;
#if FALSE
	string.sPrintf("source=%02p item=%02p state=%3d,%3d"
			" mouse=%4d,%4d mb=%p",
			event.source(),event.item(),event.state(),event.state2(),
			event.mouseX(),event.mouseY(),event.mouseButtons());
#endif
	string.sPrintf("src ");
	switch(event.source())
	{
		case WindowEvent::e_sourceNull:
			string.cat("NUL ");
			break;
		case WindowEvent::e_sourceMousePosition:
			string.cat("POS ");
			break;
		case WindowEvent::e_sourceMouseButton:
			string.cat("MB  ");
			break;
		case WindowEvent::e_sourceKeyboard:
			string.cat("KEY ");
			break;
		case WindowEvent::e_sourceSystem:
			string.cat("SYS ");
			break;
		case WindowEvent::e_sourceUser:
			string.cat("USR ");
			break;
		case WindowEvent::e_sourceJoy0:
			string.cat("JOY ");
			break;
		case WindowEvent::e_sourceAny:
			string.cat("ANY ");
			break;
		default:
			string.cat("%02p ",(U32)event.source());
	}

	string.cat("item ");

	if(event.item()==WindowEvent::e_itemNull)
		string.cat("NULL  ");
	else if(event.item()==WindowEvent::e_itemAny)
		string.cat("ANY   ");
	else
	{
		String modifiers;
		U32 item2=event.item();
		BWORD cryptic=FALSE;
		if(item2&WindowEvent::e_keyControl)
		{
			item2^=WindowEvent::e_keyControl;
			modifiers.cat("C");
//				cryptic=TRUE;
		}
		else
			modifiers.cat(" ");
		if(item2&WindowEvent::e_keyShift)
		{
			item2^=WindowEvent::e_keyShift;
			modifiers.cat("S");
			cryptic=(item2<33 || item2>127);
		}
		else
			modifiers.cat(" ");
		if(item2&WindowEvent::e_keyCapLock)
		{
			item2^=WindowEvent::e_keyCapLock;
			modifiers.cat("L");
		}
		else
			modifiers.cat(" ");

		switch(event.source())
		{
			case WindowEvent::e_sourceMousePosition:
				switch(item2)
				{
					case WindowEvent::e_itemMove:
						string.cat("%sMV ",modifiers.c_str());
						break;
					case WindowEvent::e_itemDrag:
						string.cat("%sDR ",modifiers.c_str());
						break;
					default:
						string.cat("%03p ",(U32)event.item());
						break;
				}
				break;
			case WindowEvent::e_sourceMouseButton:
				switch(item2)
				{
					case WindowEvent::e_itemLeft:
						string.cat("%sLT ",modifiers.c_str());
						break;
					case WindowEvent::e_itemMiddle:
						string.cat("%sMD ",modifiers.c_str());
						break;
					case WindowEvent::e_itemRight:
						string.cat("%sRT ",modifiers.c_str());
						break;
					case WindowEvent::e_itemWheel:
						string.cat("%sWH ",modifiers.c_str());
						break;
					default:
						string.cat("%03p ",(U32)event.item());
						break;
				}
				break;
			case WindowEvent::e_sourceKeyboard:
			{
				string.cat(modifiers);
				if(item2==WindowEvent::e_keyBackspace)
					string.cat("BS ",item2);
				else if(item2==WindowEvent::e_keyTab)
					string.cat("HT ",item2);
				else if(item2==WindowEvent::e_keyLineFeed)
					string.cat("LF ",item2);
				else if(item2==WindowEvent::e_keyCarriageReturn)
					string.cat("CR ",item2);
				else if(item2==WindowEvent::e_keySpace)
					string.cat("SP ",item2);
				else if(item2==WindowEvent::e_keyEscape)
					string.cat("ES ",item2);
				else if(item2==WindowEvent::e_keyDelete)
					string.cat("DL ",item2);
				else if(!cryptic && item2<=WindowEvent::e_keyAsciiMask)
					string.cat("%c  ",item2);
				else if(!cryptic && (item2&WindowEvent::e_keyCursorMask) &&
						!(item2&(~WindowEvent::e_keyCursorMask)))
				{
					static char direction[4][6]=
							{"UP ","DN ","LT ","RT "};
					string.cat("%s%s ",modifiers.c_str(),
							direction[(item2-1)&3]);
				}
				else
					string.cat("%03p ",(U32)item2);
			}
				break;
			case WindowEvent::e_sourceSystem:
				switch(event.item())
				{
					case WindowEvent::e_itemIdle:
						string.cat("IDLE  ");
						break;
					case WindowEvent::e_itemResize:
						string.cat("RSIZE ");
						break;
					case WindowEvent::e_itemExpose:
						string.cat("EXPOS ");
						break;
					case WindowEvent::e_itemCloseWindow:
						string.cat("CLOSE ");
						break;
					case WindowEvent::e_itemQuit:
						string.cat("QUIT  ");
						break;
					case WindowEvent::e_itemPoll:
						string.cat("POLL  ");
						break;
					default:
						string.cat("%05p ",(U32)event.item());
						break;
				}
				break;
			case WindowEvent::e_sourceJoy0:
				switch(event.item())
				{
					case WindowEvent::e_joyLeftStickX:
						string.cat("L SX  ");
						break;
					case WindowEvent::e_joyLeftStickY:
						string.cat("L SY  ");
						break;
					case WindowEvent::e_joyLeftPadX:
						string.cat("L PX  ");
						break;
					case WindowEvent::e_joyLeftPadY:
						string.cat("L PY  ");
						break;
					case WindowEvent::e_joyRightStickX:
						string.cat("R SX  ");
						break;
					case WindowEvent::e_joyRightStickY:
						string.cat("R SY  ");
						break;

					case WindowEvent::e_joyStart:
						string.cat("START ");
						break;
					case WindowEvent::e_joySelect:
						string.cat("SELCT ");
						break;
					case WindowEvent::e_joyLeftHigh:
						string.cat("L HI  ");
						break;
					case WindowEvent::e_joyLeftLow:
						string.cat("L LO  ");
						break;
					case WindowEvent::e_joyLeftDepress:
						string.cat("L DEP ");
						break;
					case WindowEvent::e_joyRightUp:
						string.cat("R UP  ");
						break;
					case WindowEvent::e_joyRightDown:
						string.cat("R DN  ");
						break;
					case WindowEvent::e_joyRightLeft:
						string.cat("R LT  ");
						break;
					case WindowEvent::e_joyRightRight:
						string.cat("R RT  ");
						break;
					case WindowEvent::e_joyRightHigh:
						string.cat("R HI  ");
						break;
					case WindowEvent::e_joyRightLow:
						string.cat("R LO  ");
						break;
					case WindowEvent::e_joyRightDepress:
						string.cat("R DEP ");
						break;

					default:
						string.cat("%05p ",(U32)event.item());
						break;
				}
				break;
			case WindowEvent::e_sourceUser:
				switch(event.item())
				{
					case WindowEvent::e_windowClear:
						string.cat("CLEAR ");
						break;
					case WindowEvent::e_windowSwapBuffers:
						string.cat("SWAP  ");
						break;
					default:
						string.cat("%05p ",(U32)event.item());
						break;
				}
				break;
			default:
				string.cat("%08p ",(U32)event.item());
		}
	}

	string.cat("state ");
	if(event.source()==WindowEvent::e_sourceMouseButton &&
			event.item()==WindowEvent::e_itemWheel)
	{
		string.cat("%-5d ",(I32)event.state());
	}
	else
	{
		switch(event.source())
		{
			case WindowEvent::e_sourceMouseButton:
			case WindowEvent::e_sourceKeyboard:
			case WindowEvent::e_sourceMousePosition:
				switch(event.state())
				{
					case WindowEvent::e_stateNull:
						string.cat("NULL  ");
						break;
					case WindowEvent::e_statePress:
						string.cat("PRESS ");
						break;
					case WindowEvent::e_stateRelease:
						string.cat("REL   ");
						break;
					case WindowEvent::e_stateRepeat:
						string.cat("REP+  ");
						break;
					case WindowEvent::e_stateAny:
						string.cat("ANY   ");
						break;
					default:
						string.cat("%-5d ",(I32)event.state());
						break;
				}
				break;
			default:
				string.cat("%-5d ",(I32)event.state());
		}
	}

	if(event.state2())
		string.cat(",%-4d ",(I32)event.state2());
	else
		string.cat("      ");

	string.cat("at %4d,%4d",event.mouseX(),event.mouseY());

	if(event.mouseButtons())
		string.cat(" mb %p",event.mouseButtons());
	else
		string.cat("       ");

	return string;
}

} /* namespace */
