/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __vegetation_Forest_h__
#define __vegetation_Forest_h__

FE_ATTRIBUTE("sim:effector",	"Spherical obstacle (x,y,z,r)");
FE_ATTRIBUTE("sim:effectForce",	"Force on effector");

namespace fe
{

/**************************************************************************//**
	@brief Forest RecordView

	@ingroup vegetation
*//***************************************************************************/
class FE_DL_EXPORT Forest: public RecordView
{
	public:
		Functor<Vector4>			effector;
		Functor<SpatialVector>		effectorVelocity;
		Functor<SpatialVector>		effectForce;
		Functor<SpatialVector>		windSource;
		Functor<SpatialVector>		windDirection;
		Functor< sp<RecordGroup> >	plants;
		Functor< sp<Component> >	pointI;
		Functor< sp<Component> >	drawI;
		Functor< sp<Component> >	controllerI;
		Functor<F32>				time;
		Functor<F32>				deltaT;

				Forest(void)		{ setName("Forest"); }
virtual	void	addFunctors(void)
				{
					add(effector,	"vector4",		FE_USE("sim:effector"));
					add(effectorVelocity,"spatial_vector",
													FE_USE("sim:effectorVel"));
					add(effectForce,"spatial_vector",
													FE_USE("sim:effectForce"));
					add(windSource,"spatial_vector",
													FE_USE("sim:windSource"));
					add(windDirection,"spatial_vector",
													FE_USE("sim:windDir"));

					add(plants,		"RecordGroup",
							FE_SPEC("veg:plants","RecordGroup of vegetation"));

					add(pointI,		"Component",	FE_USE("ren:PointI"));
					add(drawI,		"Component",	FE_USE("ren:DrawI"));
					add(controllerI,"Component",	FE_USE("ren:ControllerI"));

					add(time,		"F32",			FE_USE("sim:time"));
					add(deltaT,		"F32",			FE_USE("sim:timestep"));
				}
virtual	void	initializeRecord(void)
				{
					plants.createAndSetRecordGroup();

					set(effector());
					set(effectorVelocity());
					set(effectForce());
					set(windSource());
					set(windDirection());

					time()=0.0f;
					deltaT()=(1.0f/60.0f);
				}
};

} /* namespace */

#endif /* __vegetation_Forest_h__ */
