/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __data_AccessorSet_h__
#define __data_AccessorSet_h__

namespace fe
{

/** @brief Set of accessors

	@ingroup data
*/

class FE_DL_EXPORT AccessorSet :
	virtual public PopulateI
{
	public:
						AccessorSet(void)
						{
							setActive(false);
						}
virtual					~AccessorSet(void)
						{
						}
virtual	UWORD			size(void) const { return m_accessors.size(); }
virtual	BaseAccessor	*operator[](UWORD index)
						{
							return m_accessors[index];
						}


		bool	bind(const WeakRecord a_record)
		{
			if(!a_record.isValid()) { return false; }
			bind(a_record.layout()->scope());
			return check(a_record);
		}
		void	bind(sp<Scope> spScope)
		{
			if(!spScope.isValid())
			{
				feX("AccessorSet::bind",
					"given invalid scope");
			}
			if(m_hpScope==spScope)
			{
				return;
			}
			if(m_hpScope.isValid())
			{
				feX("AccessorSet::bind","changed scope\n");
			}
			m_hpScope = spScope;

			setActive(true);
			initializeAll();
		}
virtual	void	populate(sp<Layout> spLayout)
		{
			if(!spLayout.isValid())
			{
				feX("AccessorSet::populate","has invalid layout\n");
			}
			bind(spLayout->scope());
			for(unsigned int i = 0; i < size(); i++)
			{
				spLayout->populate(*((*this)[i]));
			}
		}
		void	enforceHaving(const AccessorSet &a_other)
		{
			if(!m_hpScope.isValid())
			{
				feX("AccessorSet::populate","unset scope\n");
			}

			std::vector<String> self_set;
			for(unsigned int i = 0; i < size(); i++)
			{
				self_set.push_back((*this)[i]->name());
			}

			for(unsigned int i = 0; i < a_other.size(); i++)
			{
				m_hpScope->enforce(self_set, a_other.m_accessors[i]->name());
			}
		}
		bool	bindCheck(sp<Layout> spLayout)
		{
			bind(spLayout->scope());
			for(unsigned int i = 0; i < size(); i++)
			{
				if(!((*this)[i]->check(spLayout))) { return false; }
			}
			return true;
		}
		bool	check(sp<Layout> spLayout)
		{
			for(unsigned int i = 0; i < size(); i++)
			{
				if(!((*this)[i]->check(spLayout))) { return false; }
			}
			return true;
		}
		bool	bindCheck(sp<RecordArray> spRA)
		{
			bind(spRA->layout()->scope());
			for(unsigned int i = 0; i < size(); i++)
			{
				if(!((*this)[i]->check(spRA))) { return false; }
			}
			return true;
		}
		bool	check(sp<RecordArray> spRA)
		{
			for(unsigned int i = 0; i < size(); i++)
			{
				if(!((*this)[i]->check(spRA))) { return false; }
			}
			return true;
		}
		bool	check(const WeakRecord a_record)
		{
			for(unsigned int i = 0; i < size(); i++)
			{
				if(!((*this)[i]->check(a_record))) { return false; }
			}
			return true;
		}
		void	check(const WeakRecord a_record,
					const char *a_annotation)
		{
			for(unsigned int i = 0; i < size(); i++)
			{
				if(!((*this)[i]->check(a_record)))
				{
					feX("AccessorSet::assert",
						"%s | %s | missing %s",
						(a_annotation!=NULL)?a_annotation:"",
						a_record.layout()->name().c_str(),
						(*this)[i]->attribute()->name().c_str());
				}
			}
		}
		void	enforce(const String &a_ifHas)
		{
			for(unsigned int i = 0; i < size(); i++)
			{
				m_hpScope->enforce(a_ifHas, (*this)[i]->attribute()->name());
			}
		}

		void	filter(sp<RecordGroup> rg_output, sp<RecordGroup> rg_input)
		{
			for(RecordGroup::iterator i_in = rg_input->begin();
				i_in != rg_input->end(); i_in++)
			{
				bind((*i_in)->layout()->scope());
				if(check(*i_in))
				{
					for(int i = (*i_in)->length() - 1; i >= 0; i--)
					{
						rg_output->add((*i_in)->getRecord(i));
					}
				}
			}
		}
		hp<Scope>	scope(void) { return m_hpScope; }
	protected:
		void		add(BaseAccessor &a_accessor, const String &a_attribute)
					{
						if(m_hpScope.isValid())
						{
							a_accessor.setup(m_hpScope, a_attribute);
							m_accessors.push_back(&a_accessor);
						}
					}
	protected:
		hp<Scope>											m_hpScope;
		std::vector<BaseAccessor *>							m_accessors;
};

} /* namespace */

#endif /* __data_AccessorSet_h__ */

