/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "data.pmh"

namespace fe
{

Attribute::Attribute(void)
{
	m_name = "--unset--";
	m_spType = NULL;
	m_serialize = true;
	m_cloneable = true;
	suppressReport();
}

Attribute::Attribute(const String name, sp<BaseType> type, sp<TypeMaster> spTM)
{
	if(! type.isValid())
	{
		feX("fe::Attribute::Attribute","attempt construction with NULL type");
	}
	m_spType = type;
	m_name = name;
	m_serialize = true;
	m_cloneable = true;
	m_spTypeMaster = spTM;
	suppressReport();
}


Attribute::Attribute(const Attribute &other)
{
	m_name = other.m_name;
	m_spType = other.m_spType;
	m_spTypeMaster = other.m_spTypeMaster;
	m_serialize = true;
	m_cloneable = true;
	suppressReport();
}

Attribute::~Attribute(void)
{
}

Attribute &Attribute::operator=(const Attribute &other)
{
	if(&other == this) { return *this; }
	m_name = other.m_name;
	m_spType = other.m_spType;
	m_spTypeMaster = other.m_spTypeMaster;
	m_serialize = other.m_serialize;
	m_cloneable = other.m_cloneable;
	return *this;
}

void Attribute::peek(Peeker &peeker)
{
	peeker.str().cat("%-15s %d", m_name.c_str(), m_serialize);
}

bool Attribute::isSerialize(void)
{
	return m_serialize;
}

void Attribute::setSerialize(bool set)
{
	m_serialize = set;
}

bool Attribute::isCloneable(void)
{
	return m_cloneable;
}

void Attribute::setCloneable(bool set)
{
	m_cloneable = set;
}


} /* namespace */

