/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __data_RecordCookbook_h__
#define __data_RecordCookbook_h__

namespace fe
{

/**************************************************************************//**
	@brief Instructions to set attributes of a Record

	@ingroup data
*//***************************************************************************/
class FE_DL_EXPORT RecordCookbook: public Handled<RecordCookbook>
{
	public:

	class Ingredient
	{
		public:
					Ingredient(void): m_block(NULL)						{}
					~Ingredient(void)		{ delete[] m_block; }

			void	set(UWORD offset,sp<BaseType>& rspBT,char* data);

			sp<BaseType>	m_spBT;
			UWORD			m_offset;
			char*			m_block;
	};

typedef ext::hash_map<		String,
							String,
							hash_string,
							eq_string > StringStringMap;
typedef ext::hash_map<		String,
							Ingredient,
							hash_string,
							eq_string > StringIngredientMap;

					RecordCookbook(hp<Scope> hpScope):
							m_hpScope(hpScope),
							m_name("RecordCookBook")						{}

		sp<Scope>	scope(void)		{ return m_hpScope; }

					///	@brief Add a prerequisite to a recipe
		void		derive(String recipeName,String parentName);

					/**	@brief Return the layouts for a recipe

						Often the Layout name will match the recipe name,
						but if a derived recipe is merging Layouts or
						just replacing replacing initial values, it may not.
						In this case, the parent classes are searched
						for a required Layouts. */
		void		findLayoutList(String recipeName,
							std::vector<String>& rLayoutList);

					/** @brief Define an initial value for a recipe's attribute

						Redundant definitions favor the latest. */
		void		add(String recipeName,String attributeName,String value);

					/// @brief Apply initial values for a recipe to a Record
		void		cook(String recipeName,Record& rRecord);

					/** @brief Recursively cook recipes

						@internal */
		void		cook_internal(String recipeName,Record& rRecord,
							BWORD try_cache,StringIngredientMap* pNewCacheMap);

					/** @brief Recursively clear cached Record templates

						@internal */
		void		spoil(String recipeName);

	const	String&	name(void) const		{ return m_name; }

	private:

	class Recipe: public Counted
	{
		public:
					Recipe(String name,sp<RecordCookbook> spRecordCookBook):
						m_name(name),
						m_hpRecordCookbook(spRecordCookBook)				{}

			void	derive(String recipeName);
			void	adopt(String recipeName);
			void	findLayoutList(std::vector<String>& rLayoutList);
			void	add(String attributeName,String value);
			void	cook(Record& rRecord,BWORD ignore_cache,
							StringIngredientMap* pNewCacheMap);
			void	spoil(void);

	const	String&	name(void) const		{ return m_name; }
	const	String	verboseName(void) const	{ return "Recipe " + name(); }

		private:

			String				m_name;
			hp<RecordCookbook>	m_hpRecordCookbook;
			std::vector<String>	m_prerequisites;
			std::vector<String>	m_dependents;
			StringStringMap		m_attributeMap;
			StringIngredientMap	m_cacheMap;
	};

		sp<Recipe> lookup(String recipeName);

		hp<Scope>			m_hpScope;
		String				m_name;


typedef ext::hash_map<		String,
							sp<Recipe>,
							hash_string,
							eq_string > StringRecipeMap;

		StringRecipeMap		m_recipeMap;
};

} /* namespace */

#endif /* __data_RecordCookbook_h__ */
