/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __data_SegmentStore_h__
#define __data_SegmentStore_h__

namespace fe
{

/** @brief Memory manager for state blocks.

	@ingroup data
	*/
class SegmentStore : public StoreI, public ClassSafe<GlobalHolder>
{
	public:
						SegmentStore(void);
virtual					~SegmentStore(void);
virtual	void			*createSB(void);
virtual	void			*createSB(UWORD count);
virtual	void			acquireSB(void *stateBlock);
virtual	void			releaseSB(void *stateBlock);
virtual	void			freeSB(void *stateBlock);
virtual	hp<Layout>&		getLayout(void);
virtual	void			setLayout(sp<Layout>& rspLayout);
virtual	UWORD			skipSizeSB(void);

						/// Get the long name of this Store
		const String	verboseName(void) const;

	private:
		void			initSB(void *stateBlock);
		void			clear(void);
		void*			stateBlockOf(U32 index,void* pSegment);
		UWORD			stateBlockBytes(void);
		UWORD			segmentBytes(U32 count);

		class FreeHeader
		{
			public:
				FreeHeader		*m_pNext;
		};
		class Segment
		{
			public:
				UWORD			m_allocated;
				UWORD			m_used;
				Segment			*m_pNext;
		};

		hp<Layout>				m_hpLayout;
		Segment					*m_pSegment;
		FreeHeader				*m_pFreeList;
		Accessor<I32>			m_aRC;
		UWORD					m_cntIndex;
};

inline const String SegmentStore::verboseName(void) const
{
	return "SegmentStore " + name();
}


} /* namespace */

#endif /* __data_SegmentStore_h__ */

