/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "data.pmh"

#define FE_WR_CHECK_SN	TRUE

namespace fe
{

void WeakRecord::checkSerialNumber(void) const
{
#if FE_WR_CHECK_SN
	if(isValid() && m_serialNumber<0)
	{
		feLog("WeakRecord::cacheSerialNumber layout \"%s\""
				" doesn't support \":SN\" attribute\n",
				m_hpLayout->name().c_str());
		feX("WeakRecord::checkSerialNumber","no SN");
	}
#endif
}

String WeakRecordInfo::print(void *instance)
{
	if(!instance)
		return String("<error>");

	const WeakRecord* pR = (WeakRecord*)instance;
	if(!pR->isValid())
		return String("<invalid>");

	String string;
	string.sPrintf("%p %s",pR->data(),
			pR->layout().isValid()?
			pR->layout()->name().c_str(): "<invalid layout>");
	return string;
}

IWORD WeakRecordInfo::output(std::ostream &ostrm, void *instance,
		t_serialMode mode)
{
	feX(e_unsupported,
		"WeakRecordInfo::output",
		"record output should be done via fe::Stream");
	return 0;
}

void WeakRecordInfo::input(std::istream &istrm, void *instance,
		t_serialMode mode)
{
	feX(e_unsupported,
		"WeakRecordInfo::input",
		"record input should be done via fe::Stream");
}

IWORD WeakRecordInfo::iosize(void)
{
	return c_implicit;
}

bool WeakRecordInfo::getConstruct(void)
{
	return true;
}

void WeakRecordInfo::construct(void *instance)
{
	new(instance)WeakRecord;
}

void WeakRecordInfo::destruct(void *instance)
{
	((WeakRecord *)instance)->~WeakRecord();
}

} /* namespace */

