/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "fe/data.h"

using namespace fe;

int main(void)
{
	UNIT_START();
	BWORD complete=FALSE;
	try
	{
		fe::sp<fe::Master> spMaster(new fe::Master);
		fe::sp<fe::Scope> spScope(new fe::Scope(*spMaster.raw()));

		AsSystem asSystem;
		asSystem.bind(spScope);

		sp<Layout> spLayout=spScope->declare("sack");
		spScope->support("quantity", "integer");
		spScope->populate("sack", "quantity");

		sp<Layout> spLayout2=spScope->declare("tack");
		spScope->support("bonus", "integer");
		spScope->populate("sack", "bonus");

		Accessor<I32> aQuantity;
		aQuantity.setup(spScope, "quantity", "integer");

		Record first=spScope->produceRecord("sack");
		feLog("\n");

		spScope->cookbook()->add("sack","quantity","7");

		Record second=spScope->produceRecord("sack");
		UNIT_TEST(aQuantity(second)==7);
		feLog("\n");

		spScope->cookbook()->derive("pack","sack");
		spScope->cookbook()->add("pack","quantity","3");

		Record third=spScope->produceRecord("pack");
		UNIT_TEST(aQuantity(third)==3);
		feLog("\n");

		char* filename="test/knack.rg";
		std::ifstream inFile(filename);
		if(!inFile)
		{
			feX("could not open \"%s\"\n",filename);
		}

		sp<RecordGroup> spRecordGroup;

		sp<data::StreamI> spStream(new data::AsciiStream(spScope));
		spRecordGroup=spStream->input(inFile);
		inFile.close();

		Record fourth=spScope->produceRecord("knack");
		UNIT_TEST(aQuantity(fourth)==5);
		feLog("\n");

		Record fifth=spScope->produceRecord("knack");
		UNIT_TEST(aQuantity(fifth)==5);

		Record sixth=spScope->produceRecord("rack");
		UNIT_TEST(aQuantity(sixth)==13);

		Record seventh=spScope->produceRecord("rack");
		UNIT_TEST(aQuantity(seventh)==13);

		complete=TRUE;
	}
	catch(Exception &e) { e.log(); }
	catch(...)			{ feLog("uncaught exception\n"); }

	UNIT_TEST(complete);
	UNIT_TEST(!Counted::trackerCount());
	UNIT_TRACK(8);
	UNIT_RETURN();
}
