/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __platform_h__
#define __platform_h__

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <assert.h>
#include <typeinfo>

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <list>
#include <map>
#include <set>
#include <stack>
#include <queue>

#include "fe/config.h"

#include "platform/define.h"

// hash_map is not 'standard', so we have some hoops
#if defined(HASH_MAP_IS_MAP)
namespace ext
{
	template<class k, class v, class h, class e>
	class hash_map : public std::map< k,v > { };
}
#elif defined(GNU_STL_HASH_MAP)
#include <ext/hash_map>
namespace ext = __gnu_cxx;
#elif FE_OS==FE_WIN32
#include <hash_map>
#else
#include <ext/hash_map>
#endif

#if FE_OS==FE_WIN32

	#include <CRTDbg.h>
	#include <windows.h>	//* causes wacky struct errors (???)
	#if FE_COMPILER!=FE_DMC
	#include <winsock2.h>
	#endif
	#include <mmsystem.h>
	#include <direct.h>

	#define mkdir _mkdir

	#ifndef WIN32_LEAN_AND_MEAN
		#define WIN32_LEAN_AND_MEAN
	#endif

	#define FE_MESSAGE_LENGTH_MAX					512

#elif FE_OS==FE_LINUX
	#include <sys/time.h>
	#include <sys/stat.h>
	#include <sys/types.h>
	#include <unistd.h>
	#include <errno.h>
	#include <netinet/in.h>
	#include <dlfcn.h>
	#include <sys/socket.h>
	#include <arpa/inet.h>
	#include <netdb.h>

	#define FE_MESSAGE_LENGTH_MAX					512

#elif FE_OS==FE_OSX
	#include <sys/time.h>
	#include <sys/stat.h>
	#include <sys/types.h>
	#include <unistd.h>
	#include <errno.h>
	#include <netinet/in.h>
	#include <dlfcn.h>
	#include <sys/socket.h>
	#include <arpa/inet.h>
	#include <netdb.h>
	// and I thought MS was presumptuous...check() is a macro for OSX
	#include <AssertMacros.h>
	#ifdef check
	#undef check
	#endif

	#define FE_MESSAGE_LENGTH_MAX					512

#elif FE_OS==FE_CYGWIN
	#include <sys/time.h>
	#include <sys/types.h>
	#include <unistd.h>
	#include <errno.h>
	#include <netinet/in.h>
	#include <dlfcn.h>
	#include <sys/socket.h>
	#include <arpa/inet.h>
	#include <netdb.h>

	#define FE_MESSAGE_LENGTH_MAX					512
#endif

#if FE_COMPILER==FE_GNU && FE_OS==FE_OSX
#ifndef GCC_HASCLASSVISIBILITY
#define GCC_HASCLASSVISIBILITY
#endif
#endif

#if FE_COMPILER==FE_MICROSOFT
	#define FE_DL_EXPORT	__declspec(dllexport)
	#define FE_DL_IMPORT	__declspec(dllimport)
	#define FE_DL_PUBLIC
	#define FE_DL_LOCAL
#elif FE_COMPILER==FE_GNU && defined(GCC_HASCLASSVISIBILITY)
	//* NOTE GCC manual specifies empty FE_DL_IMPORT
	#define FE_DL_EXPORT	__attribute__ ((visibility("default")))
	#define FE_DL_IMPORT
	#define FE_DL_PUBLIC	FE_DL_EXPORT
	#define FE_DL_LOCAL		__attribute__ ((visibility("hidden")))
#else
	#define FE_DL_EXPORT
	#define FE_DL_IMPORT
	#define FE_DL_PUBLIC
	#define FE_DL_LOCAL
#endif

#if FE_COMPILER==FE_GNU
	#include <new>
#elif FE_COMPILER==FE_MICROSOFT
	//* DISABLE:  'this' : used in base member initializer list
	#pragma warning( disable : 4355 )
	//* DISABLE:  dll-interface complaints
	#pragma warning( disable : 4275 )
	#pragma warning( disable : 4251 )
	#include <new.h>
#else
	#include <new.h>
#endif


#if FE_OS==FE_WIN32
	#include <mbstring.h>
#else
	#include <ctype.h>
#endif

#include "boost/version.hpp"
#include "boost/thread.hpp"

#include "platform/datatypes.h"
#include "platform/error.h"
#include "platform/memory.h"
#include "platform/String.h"
#include "platform/debug.h"
#include "platform/System.h"
#include "platform/Result.h"
#include "platform/Exception.h"
#include "platform/SystemTicker.h"
#include "platform/UnitTest.h"
#include "platform/DL_Loader.h"

#endif /* __platform_h__ */

