/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __platform_debug_h__
#define __platform_debug_h__

/**************************************************************************//**
	@brief Dump the program stack
*//***************************************************************************/
void feLogBacktrace(U32 skip=1);

/**************************************************************************//**
	@brief Register a built-in handler for SIGSEGV
*//***************************************************************************/
FE_DL_PUBLIC void feRegisterSegvHandler(void);

namespace fe
{

#if FE_CODEGEN>FE_DEBUG
#define FEASSERT(condition)
#elif FE_OS==FE_WIN32
#define FEASSERT(condition) _ASSERT(condition)
#else
#define FEASSERT(condition) assert(condition)
#endif

/**************************************************************************//**
	@brief Format text into a fe::String
*//***************************************************************************/
void vsPrintf(std::string &target, const char *fmt, va_list ap);

/**************************************************************************//**
	@brief Interface to log text
*//***************************************************************************/
class Logger
{
	public:
virtual				~Logger(void)											{}

virtual		void	operator()(const char *format, ...)	const				=0;
virtual		void	operator()(const String &string)	const				=0;
};

/**************************************************************************//**
	@brief Logger Implemention to output text directly to a console

	Under Win32, the string is also sent to the debugger using
	OutputDebugString().
*//***************************************************************************/
class ImmediateOutputLogger : public Logger
{
	public:
virtual				~ImmediateOutputLogger(void)							{}

virtual		void	operator()(const char *format, ...) const;
virtual		void	operator()(const String &string) const;
};

extern ImmediateOutputLogger debugLogger;

} /* namespace */

// This does not seem to work properly in g++ 3 if it is in a namespace
extern FE_DL_PUBLIC fe::Logger& feLog;

#endif /* __platform_debug_h__ */

