/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __platform_memory_h__
#define __platform_memory_h__

#define FE_EXPORT_MEM_VIA_FUNC

//* HACK something with amd64, multi-processor, or a bug causes malloc on stack
#ifdef __x86_64__
#define	FE_HEAP_CHECKABLE	FALSE
#else
#define	FE_HEAP_CHECKABLE	TRUE
#endif

#ifdef FE_EXPORT_MEMORY
#define FE_MEM_PORT	FE_DL_EXPORT
#else
#define FE_MEM_PORT	FE_DL_IMPORT
#endif

#ifdef FE_USE_SSE
#define FE_SSE		FE_USE_SSE
#else
#define FE_SSE		0
#endif

#if FE_SSE>=1
#define FE_MEM_ALIGNMENT	16
#else
#define FE_MEM_ALIGNMENT	0
#endif

//* boost shutdown glitch?
#if FE_OS==FE_LINUX // && FE_CODEGEN>FE_DEBUG
#define FE_SAFE_COUNTED_MUTEX	TRUE
#else
#define FE_SAFE_COUNTED_MUTEX	FALSE
#endif

namespace fe
{

extern "C"
{

typedef void*	( allocateFunction )( UWORD byteCount );
typedef void	( deallocateFunction )( void* pMemory );
typedef void*	( reallocateFunction )( void* pMemory, UWORD byteCount );

} /* extern "C" */

#if FE_COMPILER==FE_DMC
extern allocateFunction*				gs_pAllocateFunction;
extern deallocateFunction*				gs_pDeallocateFunction;
extern reallocateFunction*				gs_pReallocateFunction;
extern void*							gs_pHeapBase;
#else
FE_MEM_PORT	extern allocateFunction*	gs_pAllocateFunction;
FE_MEM_PORT	extern deallocateFunction*	gs_pDeallocateFunction;
FE_MEM_PORT	extern reallocateFunction*	gs_pReallocateFunction;
FE_MEM_PORT	extern void*				gs_pHeapBase;
#endif

FE_MEM_PORT void *ex_allocate(UWORD byteCount);
FE_MEM_PORT void *ex_reallocate(void *pMemory, UWORD byteCount);
FE_MEM_PORT void ex_deallocate(void *pMemory);

extern "C"
{

inline
void* allocate( UWORD byteCount )
{
#ifdef FE_EXPORT_MEM_VIA_FUNC
	return ex_allocate(byteCount);
#else
	return gs_pAllocateFunction( byteCount );
#endif
}


inline
void deallocate( void* pMemory )
{
//	assert( gs_pDeallocateFunction );

#ifdef FE_EXPORT_MEM_VIA_FUNC
	ex_deallocate(pMemory);
#else
	gs_pDeallocateFunction( pMemory );
#endif
}

inline
void* reallocate( void* pMemory, UWORD byteCount )
{
#ifdef FE_EXPORT_MEM_VIA_FUNC
	return ex_reallocate(pMemory, byteCount);
#else
	return gs_pReallocateFunction( pMemory, byteCount );
#endif
}

} /* extern "C" */

/**	@brief Basic memory management
	*/
class Memory
{
	public:
static	bool	isHeap(void* pPtr);

#if FE_SAFE_COUNTED_MUTEX
static	I32						ms_mutexCount;
static	boost::recursive_mutex*	ms_pMutex;
#else
static	boost::recursive_mutex	ms_mutex;
#endif
};

#if FE_OS==FE_LINUX
inline bool Memory::isHeap(void* pPtr)
{
	U8 current_stack;
//	printf("Memory::isHeap %p<%p\n",pPtr,(void*)&current_stack);
	return pPtr < (void*)&current_stack;
}
#elif FE_OS==FE_WIN32
inline bool Memory::isHeap(void* pPtr)
{
//	return _CrtIsValidHeapPointer(pPtr);
	return pPtr >= gs_pHeapBase;
}
#elif FE_OS==FE_OSX
inline bool Memory::isHeap(void* pPtr)
{
	U8 current_stack;
//	printf("Memory::isHeap %p<%p\n",pPtr,(void*)&current_stack);
	return pPtr < (void*)&current_stack;
}
#else
#error Memory::isHeap not implemented
#endif

} /* namespace */

#if TRUE//FE_OS==FE_WIN32
inline void* operator new( UWORD byteCount )
{	return fe::allocate( byteCount ); }
inline void* operator new[]( UWORD byteCount )
{	return operator new( byteCount ); }

inline void operator delete( void* pMemory )
{	fe::deallocate( pMemory ); }
inline void operator delete[]( void* pMemory )
{	operator delete( pMemory ); }
#endif

#endif /* __platform_memory_h__ */

