/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "plugin.pmh"

namespace fe
{

Master::Master(void): m_name("Master")
{
#if FE_CODEGEN<FE_OPTIMIZE
	SystemTicker::calibrate();
#endif

	m_spAllocator = new BasicAllocator();
	m_spTypeMaster = new TypeMaster();
	m_spRegistry = new Registry(this);
	assertCore(m_spTypeMaster);
	assertPlugin(m_spTypeMaster);
}

Master::Master(const Master &other): m_name("Master (copy)")
{
	if(&other != this)
	{
		feX("fe::Master::Master","clone impossible");
	}
}

Master::~Master(void)
{
#if FE_CODEGEN==FE_DEBUG
	feLog("Master Catalog:\n");
	for(std::map< String, Instance >::iterator i_scp = m_instanceMap.begin();
		i_scp != m_instanceMap.end(); i_scp++)
	{
		feLog("  %s\n", i_scp->first.c_str());
	}
	feLog("\n");
#endif
	m_spAllocator = NULL;
	m_spTypeMaster = NULL;
	m_spRegistry = NULL;
}

void Master::catalogKeys(std::vector<String> &keys)
{
	for(std::map< String, Instance >::iterator i_scp = m_instanceMap.begin();
		i_scp != m_instanceMap.end(); i_scp++)
	{
		keys.push_back(i_scp->first);
	}
}

sp<TypeMaster> Master::typeMaster(void) const
{
	return m_spTypeMaster;
}

sp<Allocator> Master::allocator(void) const
{
	return m_spAllocator;
}

sp<Registry> Master::registry(void) const
{
	return m_spRegistry;
}

bool Master::catalogLookup(const String &a_key, Instance &a_instance)
{
	std::map<String, Instance>::iterator i_v = m_instanceMap.find(a_key);
	if(i_v == m_instanceMap.end())
	{
		return false;
	}
	a_instance = i_v->second;
	return true;
}

} /* namespace */

