/*	Copyright (C) 2003-2008 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#ifndef __core_Master_h__
#define __core_Master_h__

namespace fe
{

/**************************************************************************//**
	@brief Central access point for key pseudo-global objects

	@ingroup plugin
*//***************************************************************************/
class FE_DL_EXPORT Master: public Handled<Master>
{
	public:
		Master(void);
		Master(const Master &other);
virtual	~Master(void);

		sp<TypeMaster>		typeMaster(void) const;
		sp<Allocator>		allocator(void) const;
		sp<Registry>		registry(void) const;

		void				catalogKeys(std::vector<String> &keys);
		BWORD				cataloged(String name) const;
		void				clearCatalog(void);

		sp<Component>		catalogComponent(String implementation,String name);
		sp<Component>		catalogComponent(sp<Component> instance,String name);
		Instance			&catalogInstance(Instance &instance,String name);
		Instance			&catalogInstance(String name);

		template <class T>
		T					&catalog(String name);
		template <class T>
		T					&catalog(String name, const T& a_default);

		bool				catalogLookup(const String &a_key,
								Instance &a_instance);

const	String&				name(void) const	{ return m_name; }

	private:
		String							m_name;

		sp<TypeMaster>					m_spTypeMaster;
		sp<Allocator>					m_spAllocator;
		sp<Registry>					m_spRegistry;
		std::map< String, Instance >	m_instanceMap;
};

template<class T>
T &Master::catalog(String name)
{
	if(!cataloged(name))
	{
		return m_instanceMap[name].create<T>(m_spTypeMaster);
	}
	return m_instanceMap[name].cast<T>();
}

template<class T>
T &Master::catalog(String name, const T& a_default)
{
	if(!cataloged(name))
	{
		m_instanceMap[name].create<T>(m_spTypeMaster) = a_default;
	}
	return m_instanceMap[name].cast<T>();
}


inline sp<Component> Master::catalogComponent(String implementation,
		String name)
{
	Instance &instance = m_instanceMap[name];
	sp<Component> spComponent;
	if(instance.is< sp<Component> >())
	{
		spComponent = instance.cast< sp<Component> >();
	}
	if(!spComponent.isValid())
	{
		spComponent=registry()->create(implementation,name);
		instance.create< sp<Component> >(m_spTypeMaster) = spComponent;
	}
	return spComponent;
}

inline Instance &Master::catalogInstance(String name)
{
	return m_instanceMap[name];
}

inline Instance &Master::catalogInstance(Instance &instance,String name)
{
	m_instanceMap[name]=instance;
	return m_instanceMap[name];
}

inline sp<Component> Master::catalogComponent(sp<Component> component,
		String name)
{
	Instance instance;
	instance.create< sp<Component> >(m_spTypeMaster) = component;
	m_instanceMap[name]=instance;
	return component;
}

inline BWORD Master::cataloged(String name) const
{
	return m_instanceMap.find(name)!=m_instanceMap.end();
}

inline void Master::clearCatalog(void)
{
	m_instanceMap.clear();
}



} /* namespace */

#endif /* __core_Master_h__ */

